const Backbone = require("backbone");
const _ = require("underscore");
const { ipcRenderer } = require("electron");
const {
    SessionImageModel,
    SessionSettingsModel,
    IntensityDatasetModel,
    TransmittanceDatasetModel,
    SessionComponentCollection,
    DatasetCollection
} = require("./models.js");
const spectroscopy = require("./spectroscopy.js");

let session = new SessionComponentCollection();
session.create({
    id: "smp",
    type: "image",
    title: "Sample Image"
});
session.create({
    id: "ref",
    type: "image",
    title: "Reference Image"
});
session.create({
    id: "settings",
    type: "settings"
});

let spectra = new DatasetCollection();
spectra.create({
    id: "smp",
    type: "intensity",
    title: "Sample Intensity"
});
spectra.create({
    id: "ref",
    type: "intensity",
    title: "Reference Intensity"
});
spectra.create({
    id: "transmit",
    type: "transmittance",
    title: "Transmittance"
});

let smpImage = session.get("smp");
let refImage = session.get("ref");
let settings = session.get("settings");

let smpIntensity = spectra.get("smp");
let refIntensity = spectra.get("ref");
let transmittance = spectra.get("transmit");
var data = [];
var data_smp=[];
var data_ref=[];
var data_ta=[];
var flag=0;

let imgOnChange = function (model, options) {
    let spectrum = spectra.get(this.get("id"));
    if(this.get("id")=="smp"){
      console.log("smp");
    }

    var id =this.get("id");
    let blueNM = model.get("blueNM");
    let redNM = model.get("redNM");
    console.log(redNM-blueNM);
    if (this.isReady()) {
        let specWidth = session.get("settings").get("specWidth");
        spectroscopy.calculateIntensity(this, specWidth, function (intensity) {
            spectrum.set(intensity);
            data[0]="Smp Wavelength"+","+"Smp Red"+","+"Smp G"+","+"Smp B"+","+"Wavelength"+","+"Transmittance"+","+"Absorbance"+","+"Ref Wavelength"+","+"Ref Red"+","+"Ref G"+","+"Ref B";
            if(id=="smp")
            {

            //  for (var i = 1; i < (redNM-blueNM); i++)
            //    data[i]= "\n"+intensity.nm[i]+","+intensity.r[i]+","+intensity.g[i]+","+intensity.b[i];
            //    console.log("flag==0");
            //    flag++;
            for (var i = 0; i < (redNM-blueNM); i++)
              data_smp[i]="\n"+intensity.nm[i]+","+intensity.r[i]+","+intensity.g[i]+","+intensity.b[i];
              flag++;
              console.log("smp");
              console.log(flag);
            if(flag==2)
              {
                for (var i = 0; i < (redNM-blueNM); i++)
                  data[i+1]=data_smp[i]+data_ta[i]+data_ref[i];
                ipcRenderer.send("download-file",data,0);
                data=[];
                flag--;
              }

             }
             else if (id=="ref") {
              //  for (var i = 1; i < (redNM-blueNM); i++)
              //    data[i]+= ","+intensity.nm[i]+","+intensity.r[i]+","+intensity.g[i]+","+intensity.b[i];
              //    ipcRenderer.send("download-file",data,0);
              //   //  do not forget to clear the data after everything is done

              //    data=[];
              //    flag=0;
              flag++;
              // console.log("ref");
              // console.log(flag);
                for (var i = 0; i < (redNM-blueNM); i++)
                  data_ref[i]=","+intensity.nm[i]+","+intensity.r[i]+","+intensity.g[i]+","+intensity.b[i];
                if(flag==2)
                  {
                    for (var i = 0; i < (redNM-blueNM); i++)
                      data[i+1]=data_smp[i]+data_ta[i]+data_ref[i];
                    ipcRenderer.send("download-file",data,0);
                    data=[];
                    flag--;
                  }
             }
            // console.log("imgOnChange");
            spectrum.save();
        });
    } else {
        spectrum.set({
            nm: [],
            r: [],
            g: [],
            b: [],
            sum: []
        });
        spectrum.save();

        //spectrum.get will get the properties of the model
    }
    // ipcRenderer.send("download-file",data,0);
}

smpImage.on("change", imgOnChange);
smpImage.listenTo(settings, "change:specWidth", imgOnChange);
refImage.on("change", imgOnChange);
refImage.listenTo(settings, "change:specWidth", imgOnChange);

let transmitOnChange = function (model, options) {
    console.log("transmitOnChange");
    // let blueNM = model.get("blueNM");
    // let redNM = model.get("redNM");
    this.set(spectroscopy.calculateTransmittance(smpIntensity, refIntensity));
    console.log(flag);
    if(flag==1)
    {
      var lengths=spectroscopy.calculateTransmittance(smpIntensity, refIntensity).nm.length;
      for (var i = 0; i < lengths; i++)
        data_ta[i]= ","+ spectroscopy.calculateTransmittance(smpIntensity, refIntensity).nm[i]+","+spectroscopy.calculateTransmittance(smpIntensity, refIntensity).t[i]+","+spectroscopy.calculateTransmittance(smpIntensity, refIntensity).a[i];


    }
    console.log(data);
    this.save();
}

transmittance.listenTo(smpIntensity, "change", transmitOnChange);
transmittance.listenTo(refIntensity, "change", transmitOnChange);

ipcRenderer.on("session-change", function (event) {
    console.log("Session Changed!");
    session.fetch();
});

ipcRenderer.send("compute-ready");
