const Backbone = require("backbone");
const _ = require("underscore");
const { fabric } = require("fabric-browseronly");
const { ipcRenderer } = require("electron");
const { SessionImageModel, SessionComponentCollection } = require("./models.js");
var Hammers = require("hammerjs");
// var gets=require("get-context");

// var stage=document.getElementById("Photo");
// var manager = new Hammers.Manager(stage);
// document.getElementById("Photo").style.color= "dashed";

document.getElementById("editor").style.border = "thick solid #0000FF";

function clamp (val, min, max) {
    return Math.min(max, Math.max(val, min));
}
// download function for csv file


let endpointProps = {
    radius: 20,
    originX: "center",
    originY: "center",
    hasBorders: false,
    hasControls: false,
    strokeWidth: 2
}

let coordClasses = {
    blueX: "editor-blue-x",
    blueY: "editor-blue-y",
    redX: "editor-red-x",
    redY: "editor-red-y"
}

let inputClasses = _.extend(coordClasses, {
    blueNM: "editor-blue-wvlen",
    redNM: "editor-red-wvlen"
});

let ImageEditorView = Backbone.View.extend({
    template: _.template(Backbone.$("#tmpl-editor").html()),

    initialize: function () {
        let view = this;

        view.canvasSegment = {
            line: new fabric.Rect({
                fill: "white",
                originX: "center",
                originY: "center",
                height: 1,
                hasControls: false,
                selectable: false
            }),
            blue: new fabric.Circle(_.extend(endpointProps, {
                fill: "rgba(0, 0, 255, 0.25)",
                stroke: "rgba(0, 0, 255, 1)"
            })),
            red: new fabric.Circle(_.extend(endpointProps, {
                fill: "rgba(255, 0, 0, 0.25)",
                stroke: "rgba(255, 0, 0, 1)"
            }))
        }

        fabric.Image.fromURL(view.model.get("path"), function (img) {
            view.canvasImg = img;
            let line = view.canvasSegment.line;
            let b = view.canvasSegment.blue;
            let r = view.canvasSegment.red;

            let imgSize = img.getOriginalSize();
            let imgW = imgSize.width;
            let imgH = imgSize.height;

            // We assume either all or none of the coordinates are null
            if (view.model.get("blueX") == null) {
                view.model.set({
                    blueX: Math.trunc(0.05 * imgW),
                    blueY: Math.trunc(0.5 * imgH),
                    redX: Math.trunc(0.95 * imgW),
                    redY: Math.trunc(0.5 * imgH)
                });
            }

            view.$el.html(view.template(view.model.toJSON()));
            view.$(".editor-blue-x").attr("max", imgW - 1);
            view.$(".editor-red-x").attr("max", imgW - 1);
            view.$(".editor-blue-y").attr("max", imgH - 1);
            view.$(".editor-red-y").attr("max", imgH - 1);

            let canvas = new fabric.Canvas(view.$("canvas").get(0), {
                selection: false
            });
            view.canvas = canvas;
            canvas.setBackgroundColor("#1f2020");

            img.set({
                lockUniScaling: true,
                lockRotation: true,
                centeredScaling: true,
                hasRotatingPoint: false,
                borderColor: "#0078e7",
                cornerColor: "#0078e7"
            });
            canvas.add(img).setActiveObject(img);
            canvas.add(line);
            canvas.add(b);
            canvas.add(r);

            let fitCanvas = function () {
                canvas.setWidth(view.$el.width());
                canvas.setHeight(view.$el.height());
            }
            fitCanvas();
            Backbone.$(window).resize(function (e) { fitCanvas(); });

            // Start the editor with the image fit inside the canvas.
            let fitX = canvas.getWidth() / imgW;
            let fitY = canvas.getHeight() / imgH;
            let fitXY = Math.min(fitX, fitY);
            img.setScaleX(fitXY);
            img.setScaleY(fitXY);
            img.center();
            img.setCoords();

            let canvasToImageX = function (x) {
                let imgX = img.getLeft();
                let imgSclX = img.getScaleX();
                return ((x - imgX) / imgSclX);
            }

            let canvasToImageY = function (y) {
                let imgY = img.getTop();
                let imgSclY = img.getScaleY();
                return ((y - imgY) / imgSclY);
            }

            let imageToCanvasX = function (x) {
                let imgX = img.getLeft();
                let imgSclX = img.getScaleX();
                return ((x * imgSclX) + imgX);
            }

            let imageToCanvasY = function (y) {
                let imgY = img.getTop();
                let imgSclY = img.getScaleY();
                return ((y * imgSclY) + imgY);
            }

            canvas.on("custom:lineupdate", function (ev) {
                let dx = r.getLeft() - b.getLeft();
                let dy = r.getTop() - b.getTop();
                let theta = Math.atan(dy / dx);
                let h = Math.sqrt((dx * dx) + (dy * dy));
                let midx = (r.getLeft() + b.getLeft()) / 2.0;
                let midy = (r.getTop() + b.getTop()) / 2.0;
                line.setAngle((180 / Math.PI)* theta);
                line.setLeft(midx);
                line.setTop(midy);
                line.setWidth(h);
                line.setCoords();

                canvas.renderAll();
            });

            canvas.on("custom:segupdate", function (ev) {
                let midPixel = img.getScaleX() / 2.0;
                b.setLeft(imageToCanvasX(view.model.get("blueX")) + midPixel);
                b.setTop(imageToCanvasY(view.model.get("blueY")) + midPixel);
                b.setCoords();

                r.setLeft(imageToCanvasX(view.model.get("redX")) + midPixel);
                r.setTop(imageToCanvasY(view.model.get("redY")) + midPixel);
                r.setCoords();

                canvas.trigger("custom:lineupdate");
            });

            img.on("moving", function (opts) {
                canvas.trigger("custom:segupdate");
                console.log("Move");
            });
            img.on("scaling", function (opts) {
                canvas.trigger("custom:segupdate");
                console.log("Scale");
            });


            // mc=new Hammers(img);
            //
            // mc.get('pan').set({ direction: Hammers.DIRECTION_ALL });
            // mc.on("panleft panright panup pandown tap press");

            b.on("moving", function (ev) {
                b.setLeft(
                    clamp(b.getLeft(), imageToCanvasX(0), imageToCanvasX(imgW))
                );
                b.setTop(
                    clamp(b.getTop(), imageToCanvasY(0), imageToCanvasY(imgH))
                );
                b.setCoords();

                canvas.trigger("custom:lineupdate");

                view.model.set({
                    blueX: Math.min(imgW - 1, Math.trunc(canvasToImageX(b.getLeft()))),
                    blueY: Math.min(imgH - 1, Math.trunc(canvasToImageY(b.getTop())))
                });
            });

            r.on("moving", function (ev) {
                r.setLeft(
                    clamp(r.getLeft(), imageToCanvasX(0), imageToCanvasX(imgW))
                );
                r.setTop(
                    clamp(r.getTop(), imageToCanvasY(0), imageToCanvasY(imgH))
                );
                r.setCoords();

                canvas.trigger("custom:lineupdate");

                view.model.set({
                    redX: Math.min(imgW - 1, Math.trunc(canvasToImageX(r.getLeft()))),
                    redY: Math.min(imgH - 1, Math.trunc(canvasToImageY(r.getTop())))
                });
            });

            b.on("modified", function (ev) {
                canvas.trigger("custom:segupdate");
            });

            r.on("modified", function (ev) {
                canvas.trigger("custom:segupdate");
            });

            for (let coord in coordClasses) {
                view.listenTo(view.model, "change:" + coord, function () {
                    view.$("." + coordClasses[coord]).val(view.model.get(coord));
                });
            }

            canvas.trigger("custom:segupdate");
        });
    },

    events: {
        "click .editor-params-toggle": function (e) {
            this.$(".editor-params-toggle img").toggleClass("arrow-flip");
            this.$(".editor-params-toggle").toggleClass("pure-button-active");
            this.$(".editor-params").toggle();
        },
        "click .editor-ok-button": function (e) {
            this.model.save();
            ipcRenderer.send("editor-close");
        },
        "change .editor-params input": function (e) {
            let changedVal = _.findKey(inputClasses, function (c) {
                return e.target.className.indexOf(c) !== -1;
            });

            if (changedVal) {
                let $input = this.$(e.target);

                if (e.target.checkValidity()) {
                    this.model.set(changedVal, Number($input.val()));
                    this.canvas.trigger("custom:segupdate");
                } else {
                    $input.val(this.model.get(changedVal));
                }
            }
        }
    }
});

ipcRenderer.once("editor-model-attrs", function (event, attrs) {
    let session = new SessionComponentCollection();
    session.add(attrs);
    let imgView = new ImageEditorView({ model: session.get(attrs.id), el: "#editor" });
});
