const { app, BrowserWindow, ipcMain, dialog } = require("electron");
const _ = require("underscore");
const fs = require('fs');
const {
    SessionImageModel,
    SessionSettingsModel,
    IntensityDatasetModel,
    TransmittanceDatasetModel,
    SessionComponentCollection,
    DatasetCollection
} = require("./models.js");

//require is like importing
//let initializes a value

let isDevMode = _.indexOf(process.argv, "--dev") != -1;
let computeWindow = null;
let mainWindow = null;

//opens the app only if the platform is darwin
app.on("window-all-closed", function () {
    if (process.platform !== "darwin") {
        app.quit();
    }
});

app.on("ready", function () {
    mainWindow = new BrowserWindow({show: false});
    mainWindow.setMenuBarVisibility(false);
    mainWindow.on("closed", function () {
        computeWindow.destroy();
        computeWindow = null;
        mainWindow = null;
    });
//initially when the app is ready : open the compute Window where you enter the parameters
    computeWindow = new BrowserWindow({show: isDevMode});
    computeWindow.loadURL("file://" + __dirname + "/compute.html");
});

//after its compute-ready then open index.html
ipcMain.once("compute-ready", function () {
    mainWindow.show();
    mainWindow.loadURL("file://" + __dirname + "/index.html");
});

/*
    Backbone uses RESTful endpoints, where every model is associated with a
    resource URL, to persist model state. We overload Backbone.sync so that
    that state is persisted in this global store object keyed by the URL,
    using Electron IPC (see src/models.js) to pass the stringified JSON.
*/
let store = {
    "session": [],
    "data": []
};

let storeTraverse = function (subStore, urlParts) {
    if (urlParts.length == 0) {
        return undefined;
    }

    let root = urlParts.shift();

    if (_.isArray(subStore)) {
        let childIdx = _.findIndex(subStore, function (model) {
            return model.id == root;
        });

        if (childIdx != -1 && urlParts.length != 0) {
            return storeTraverse(child, urlParts);
        } else {
            return { array: subStore, index: childIdx };
        }
    } else {
        if (subStore[root] && urlParts.length != 0) {
            return storeTraverse(subStore[root], urlParts);
        } else {
            return { object: subStore, key: root };
        }
    }
}

let urlParts = function (url) {
    let parts = url.split("/");
    if (parts[0] === "") {
        parts.shift();
    }
    return parts;
}

ipcMain.on("backbone-get", function (event, url, channel) {
    let t = storeTraverse(store, urlParts(url));
    if (t && t.object && t.object[t.key]) {
        event.sender.send(channel, t.object[t.key]);
    } else if (t && t.array && t.index != -1) {
        event.sender.send(channel, t.array[t.index]);
    } else {
        event.sender.send(channel, "404");
    }
});

ipcMain.on("backbone-set", function (event, url, channel, val) {
    let parts = urlParts(url);

    let t = storeTraverse(store, parts);
    if (t && t.object) {
        t.object[t.key] = val;
    } else if (t && t.array) {
        if (t.index != -1) {
            t.array[t.index] = val;
        } else {
            t.array.push(val);
        }
    }
    event.sender.send(channel);

    parts = urlParts(url);
    if (parts[0] === "session") {
        computeWindow.webContents.send("session-change");
    }

    if (parts[0] === "data") {
        mainWindow.webContents.send("data-change");
    }
});

ipcMain.on("backbone-delete", function (event, url, channel) {
    let t = storeTraverse(store, urlParts(url));
    if (t && t.object) {
        if (t.object[t.key]) {
            delete t.object[t.key];
        }
    } else if (t && t.array) {
        if (t.index != -1) {
            t.array.splice(t.index, 1);
        }
    }
    event.sender.send(channel);
});

// url is for the Backbone model who called (a SessionImageModel)
ipcMain.on("open-file", function (event, url) {
    dialog.showOpenDialog(mainWindow, {
        properties: ["openFile"],
        filters: [
            {name: "Images", extensions: ["jpg", "png"]}
        ]
    }, function (files) {
        if (files && files[0]) {
            event.sender.send("open-file:" + url, files[0]);
        }
    });
});


ipcMain.on("download-file", function (event,data_s,flag) {
var file_name;
var save_data;
if(flag==0)
{
  data=data_s;
}
else if(flag==1)
{
dialog.showSaveDialog({ filters: [

     { name: 'csvs', extensions: ['csv'] }

    ]}, function (fileName) {

    if (fileName === undefined) return;

    fs.writeFile(fileName, data, function(err) {
        if(err) {
            return console.log(err);
            }

            console.log("The file was saved!");

});

  });
}
// readFile("/Users/richam112/Downloads/exp-1.csv");
//      function readFile(filepath){
//        fs.appendFile(filepath, data, function(err) {
//    if(err) {
//        return console.log(err);
//    }
//
//    console.log("The file was saved!");
//
// });
//      }

});

// ipcMain.on("open-csv-file", function (event) {
//   dialog.showOpenDialog(function (fileNames) {
//            // fileNames is an array that contains all the selected
//          if(fileNames === undefined){
//                  console.log("No file selected");
//          }else{
//                 ipcRenderer.send("download-file","Hello");
//          }
//      });
// });


ipcMain.on("editor-open", function (event, attrs) {
    mainWindow.webContents.once("did-finish-load", function (event) {
        mainWindow.webContents.send("editor-model-attrs", attrs);
    });
    mainWindow.webContents.loadURL("file://" + __dirname + "/editor.html");
});

ipcMain.on("editor-close", function (event) {
    mainWindow.webContents.goBack();
});
